package vehicle;

import vehicle.part.brake.Brake;
import vehicle.part.engine.Engine;
import vehicle.part.frame.Frame;
import vehicle.part.gearbox.GearBox;
import vehicle.part.gearbox.GearBoxAutomatic;
import vehicle.part.gearbox.GearBoxManual;
import vehicle.part.option.Option;
import vehicle.part.pedal.Pedal;
import vehicle.part.pedal.PedalAccelerator;
import vehicle.part.pedal.PedalBrake;
import vehicle.part.pedal.PedalClutch;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

public class Vehicle {

    private Double price;
    private String name;
    private VehicleBrand brand;

    // Parts
    private Frame frame;
    private List<Pedal> pedals;
    private GearBox gearBox;
    private List<Brake> brakes;
    private Engine engine;
    private List<Option> options;

    // Physic
    private Double velocity;


    public Vehicle(VehicleModel model){

        brand = model.getBrand();
        name = model.getName();

        // CREATING FRAME
        frame = new Frame(model.getFrameType());

        // CREATING ENGINE
        engine = new Engine("", 0d); // TODO Reprendre constructeur engine

        // CREATING gearBox
        if("MANUAL".equals(model.getEngineModel().getGearBoxType())){
            gearBox = new GearBoxManual(model.getEngineModel().getMaxGear());
        }else {
            gearBox = new GearBoxAutomatic(model.getEngineModel().getMaxGear());
        }

        // CREATING BRAKES
        brakes = new ArrayList<>();
        for(Integer i = 0; i < model.getNbOfWheels(); i++ ){
            brakes.add(new Brake(model.getBrakesType()));
        }

        // CREATING PEDALS
        pedals = new ArrayList<>();
        pedals.add(new PedalAccelerator(engine));
        pedals.add(new PedalBrake(brakes));
        if("MANUAL".equals(model.getEngineModel().getGearBoxType())) {
            pedals.add(new PedalClutch((GearBoxManual) gearBox));
        }

        // CREATING OPTIONS
        options = new ArrayList<>();

    }

    // METHODS /////////////////////////////////////////////////////////////////////////////////////////////////////////

    public void update(){

    }

    // Print the name of the car with the options and the price.
    // public String toString()
    // return "\t" + this.brandName + "is a " + name + " with that options : + " ?? " + ". Price : " + price.

    // Add an option
    // public void addOption(opt : vehicle.part.option)

    
    // GETTER //////////////////////////////////////////////////////////////////////////////////////////////////////////

    public Double getHeight(){
        return frame.getHeight();
    }

    public Double getWidth(){
        return frame.getWidth();
    }

    public Double getLength(){
        return frame.getLength();
    }

    public Double getweight(){
        return frame.getWeight();
    }

    public Frame getFrame() {
        return frame;
    }

    public List<Pedal> getPedals() {
        return pedals;
    }

    public GearBox getGearBox() {
        return gearBox;
    }

    public List<Brake> getBrakes() {
        return brakes;
    }

    public Engine getEngine() {
        return engine;
    }

    public List<Option> getOptions() {
        return options;
    }

    // DISPLAY /////////////////////////////////////////////////////////////////////////////////////////////////////////
    public void displayVehicleInformation(){
        // Clear screen
        System.out.print("\033[H\033[2J");
        System.out.flush();

        System.out.println("|------------------------------------------------------------------|");
        System.out.println("|                       VEHICULE INFORMATION                       |");
        System.out.println("|------------------------------------------------------------------|");
        System.out.println(" BRAND: " + brand + " MODEL: " + name +" PRICE: " + price);

        System.out.println("|-----------------------------------------------------------------|");
    }


    // Get the brand of the car
    public VehicleBrand getBrand(){return this.brand;}

    // Get the options of the car
    // public List<vehicle.part.option> getOptions()

    // Get the price
    public double getPrice(List<Option> options){
        int index = 0;
        double price = 0;

        if (options.size() == 0) {
            return price;
        }
        else{
            //    price = options.Stream.forEach(o -> o.getPrice);
            price = options.stream().collect(Collectors.summingDouble(Option::getPrice));
        }

        return price;

    }




}
