import garage.Task;
import garage.TestBench;
import org.junit.Before;
import org.junit.Test;
import vehicle.Vehicle;
import vehicle.VehicleModel;
import vehicle.part.pedal.PedalAccelerator;

import java.util.Arrays;

import static junit.framework.TestCase.assertEquals;
import static junit.framework.TestCase.assertTrue;
import static org.junit.Assert.assertNull;

public class TestBenchTest {

    private TestBench testBench;
    private Vehicle fiat500;
    private Vehicle peugeot5008;
    private Task task;

    @Before
    public void setUp(){
        testBench = new TestBench();

        fiat500 = new Vehicle(VehicleModel.FIAT_500);
        peugeot5008 = new Vehicle(VehicleModel.PEUGEOT_5008);
        task = new Task(fiat500.getPedals().get(0), "setState", Arrays.asList(50d), 5);
    }

    @Test
    public void addVehicleTest(){
        assertNull(testBench.getVehicle());
        testBench.addVehicle(fiat500);
        assertEquals(fiat500, testBench.getVehicle());
    }

    @Test
    public void addVehicleImpossibleTest(){
        assertNull(testBench.getVehicle());
        testBench.addVehicle(fiat500);
        testBench.addVehicle(peugeot5008);
        assertEquals(fiat500, testBench.getVehicle());
    }

    @Test
    public void removeVehicleTest(){
        testBench.addVehicle(fiat500);
        assertEquals(fiat500, testBench.getVehicle());
        testBench.removeVehicle();
        assertNull(testBench.getVehicle());
    }

    @Test
    public void removeVehicleImpossibleTest(){
        testBench.removeVehicle();
        assertNull(testBench.getVehicle());
    }

    @Test
    public void swapVehicleTest(){
        testBench.addVehicle(fiat500);
        assertEquals(fiat500, testBench.getVehicle());
        Vehicle swapped = testBench.swapVehicle(peugeot5008);
        assertEquals(fiat500, swapped);
        assertEquals(peugeot5008, testBench.getVehicle());
    }

    @Test
    public void swapVehicleImpossibleTest(){
        Vehicle swapped = testBench.swapVehicle(peugeot5008);
        assertNull(swapped);
        assertNull(testBench.getVehicle());
    }

    @Test
    public void addTaskTest(){
        assertEquals(0, testBench.getTasks().size());
        testBench.addTask(task);
        assertEquals(1, testBench.getTasks().size());
        assertTrue(testBench.getTasks().contains(task));
    }

    @Test
    public void removeTaskTest(){
        testBench.addTask(task);
        assertEquals(1, testBench.getTasks().size());
        testBench.removeTask(task);
        assertEquals(0, testBench.getTasks().size());
    }


}
